%cojoc(at)uga.edu
%http://cs.uga.edu/~hanbo/
%2012
%average mutual information between multiple networks
%   Assumption: 0*log(0)=0
%input:
%S: similarity matrix, should be possitive, the larger, the more similiar
%	S(:,:,i) similarity matrix of network i
%L: Histogram bin numbers (default:256)
%output:
%nmi: average normalized mutual information
%mi: average mutual information
%nmi_all: pairwise normalized mutual information between two images
%mi_all: pairwise normalized mutual information between two images

function [nmi mi nmi_all mi_all] = connGetNormalizedMutualInfo(S,L)
    if(min(min(S))<0)
        error('error: similarity should be larger or equal to 0.');
    end
    if(size(S,3)<2)
        error('error: input network similarity map size is less than 1.');
    end
    if nargin<2
        L=256;
    end
    S=double(S);
    n=size(S,3);
    
    %obtain histogram and entropy of each subject
    Subs=cell(size(S,3),1);
    for i=1:n
        tmp=S(:,:,i);
        Subs{i}.p=myhist(tmp(:),L);
        Subs{i}.p=Subs{i}.p/sum(Subs{i}.p);
        Subs{i}.e=entro(Subs{i}.p);
    end
    
    %calculate normalized mutual Info and mutual info
    mi_all=zeros(n);
    nmi_all=zeros(n);
    for i=1:(n-1)
        for j=i+1:n
            p2 = hist2(S(:,:,i),S(:,:,j),L); 
            p2 = p2/sum(p2(:));
            mi_all(i,j)=sum(minf(p2,Subs{i}.p'*Subs{j}.p)); 
            nmi_all(i,j)=2*mi_all(i,j)/(Subs{i}.e+Subs{j}.e);
        end
    end
    mi=sum(mi_all(:))/(n*(n-1)/2);
    nmi=sum(nmi_all(:))/(n*(n-1)/2);
    mi_all=mi_all+mi_all';
    nmi_all=nmi_all+nmi_all';
end

% -----------------------
function e=entro(p)
    I=find(p>1e-12);
    e=-sum(p(I).*log2(p(I)));
end

function y=minf(pab,papb)
    %I=find(papb(:)>1e-12 & pab(:)>1e-12); % function support 
    %I= pab(:)>1e-6;
    I=find(papb(:)>1e-12 & pab(:)>1e-12); % function support 
    y=pab(I).*log2(pab(I)./papb(I));
end

function n=myhist(A,L)
ma=min(A(:)); 
MA=max(A(:)); 
A=round((A-ma)*(L-1)/(MA-ma+eps)); 
x=0:L-1; 
n=histc(A,x,1);
n=n';
end

function n=hist2(A,B,L) 
%HIST2 Calculates the joint histogram of two images or signals
%
%   n=hist2(A,B,L) is the joint histogram of matrices A and B, using L
%   bins for each matrix.
%
%   See also MI, HIST.

%   jfd, 15-11-2006, working
%        27-11-2006, memory usage reduced (sub2ind)
%        22-10-2008, added support for 1D matrices
%        01-09-2009, commented specific code for sensorimotor signals
%        24-08-2011, speed improvements by Andrew Hill

ma=min(A(:)); 
MA=max(A(:)); 
mb=min(B(:)); 
MB=max(B(:));

% For sensorimotor variables, in [-pi,pi] 
% ma=-pi; 
% MA=pi; 
% mb=-pi; 
% MB=pi;

% Scale and round to fit in {0,...,L-1} 
A=round((A-ma)*(L-1)/(MA-ma+eps)); 
B=round((B-mb)*(L-1)/(MB-mb+eps)); 
n=zeros(L); 
x=0:L-1; 
for i=0:L-1 
    n(i+1,:) = histc(B(A==i),x,1); 
end
end
